﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;

namespace ExpandRecordUsingStyle
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            xamDataGrid1.DataSource = new ObservableCollection<Data>
                {
                    new Data 
                    {
                        Label = "Parent 1",
                        CanExpand  = false,
                        Value = 10,
                        Children = new ObservableCollection<ChildData>
                        {
                            new ChildData { ID = 1, Name = "Child 1" }, 
                            new ChildData { ID = 2, Name = "Child 2" }, 
                        }
                    },
                    new Data 
                    {
                        Label = "Parent 2",
                        CanExpand  = true,
                        Value = 22,
                        Children = new ObservableCollection<ChildData>
                        {
                            new ChildData { ID = 3, Name = "Child 3" }, 
                            new ChildData { ID = 4, Name = "Child 4" }, 
                        }
                    },
                };
        }

        private void xamDataGrid1_InitializeRecord(object sender, Infragistics.Windows.DataPresenter.Events.InitializeRecordEventArgs e)
        {
           
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            xamDataGrid1.Records[0].IsExpanded = !xamDataGrid1.Records[0].IsExpanded;
        }

        private void button1_Click_1(object sender, RoutedEventArgs e)
        {
            foreach (Data data in (xamDataGrid1.DataSource as ObservableCollection<Data>))
            {
                foreach (ChildData c in data.Children)
                {
                    c.IsHidden = true;
                }
            }
        }

    }

    public class IsHiddenToHeightConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((bool)value)
                return double.NaN;
            else
                return 0;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }


    public class IsExpandedConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((values[1] as DataRecordPresenter).Record is DataRecord
                && ((values[1] as DataRecordPresenter).Record as DataRecord).DataItem != null
                && (values[1] as DataRecordPresenter).Record.NestingDepth == 0)
            {
                DataRecordPresenter record = values[1] as DataRecordPresenter;
                if ((bool)values[0])
                {
                    record.Record.IsExpanded = true;
                }
                else
                {
                    record.Dispatcher.BeginInvoke(new Action(() =>
                    {
                        record.Record.IsExpanded = false;
                    }), System.Windows.Threading.DispatcherPriority.Background, null);
                }
               
                return Binding.DoNothing;
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class Data : INotifyPropertyChanged
    {
        private bool canExpand;
        private int value;
        private string label;
        private ObservableCollection<ChildData> children;

        public string Label
        {
            get { return label; }
            set
            {
                label = value;
                PropChanged("Label");
            }
        }

        public bool CanExpand
        {
            get { return canExpand; }
            set
            {
                canExpand = value;
                PropChanged("CanExpand");
            }
        }

        public int Value
        {
            get { return value; }
            set
            {
                this.value = value;
                PropChanged("Value");
            }
        }

        public ObservableCollection<ChildData> Children
        {
            get { return children; }
            set
            {
                this.children = value;
                PropChanged("Children");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    public class ChildData : INotifyPropertyChanged
    {
        public int ID { get; set; }
        public string Name { get; set; }

        private bool isHidden = true;

        public bool IsHidden
        {
            get { return isHidden; }
            set { isHidden = value; PropChanged("IsHidden"); }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

}
